#include "application.h"
#include "plotDialog.h"
#include "colorBox.h"
#include "colorButton.h"
#include "patternBox.h"
#include "graph.h"
#include "worksheet.h"
#include "parser.h"
#include "Histogram.h"
#include "VectXYXYCurve.h"
#include "ErrorBar.h"

#include <qaccel.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qregexp.h>
#include <qcolordialog.h>
#include <qpopupmenu.h>

plotDialog::plotDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "plotDialog" );
    setCaption( tr( "QtiPlot - Custom curves" ) );

	QVBox *box1 = new QVBox(this);
	box1->setSpacing (5);

    listBox = new QListBox( box1, "listBox" );
   	
	QHBox *hbox1 = new QHBox ( box1);
	hbox1->setSpacing (5);

    new QLabel(tr( "Plot type" ), hbox1, "TextLabel111",0 );
	boxPlotType = new QComboBox( FALSE, hbox1, "boxPlotType" );
	
	btnAssociations = new QPushButton(box1, "btnAssociations" );
    btnAssociations->setText( tr( "&Plot Associations..." ) );
	btnAssociations->hide();

	btnEditFunction = new QPushButton(box1, "btnEditFunction" );
    btnEditFunction->setText( tr( "&Edit Function..." ) );
	btnEditFunction->hide();

	QVBox  *box2=new QVBox (this); 
	box2->setSpacing (5);
    privateTabWidget = new QTabWidget(box2, "privateTabWidget" );

	initLinePage();
	initSymbolsPage();
	initHistogramPage();
	initErrorsPage();
	initSpacingPage();
	initVectPage();

	clearTabWidget();
	graph = 0;

	QHBox *hbox2 = new QHBox ( box2);
	hbox2->setSpacing (5);
	
	btnWorksheet = new QPushButton(hbox2, "btnWorksheet" );
    btnWorksheet->setText( tr( "&Worksheet" ) );
    btnWorksheet->setAutoDefault( TRUE );

	buttonApply = new QPushButton(hbox2, "buttonApply" );
    buttonApply->setText( tr( "&Apply" ) );
    buttonApply->setAutoDefault( TRUE );

    buttonOk = new QPushButton(hbox2, "buttonOk" );
    buttonOk->setText( tr( "&OK" ) );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );

    buttonCancel = new QPushButton(hbox2, "buttonCancel" );
    buttonCancel->setText( tr( "&Cancel" ) );
    buttonCancel->setAutoDefault( TRUE );
	
	QHBoxLayout* hlayout3 = new QHBoxLayout(this,5,5, "hlayout3");
    hlayout3->addWidget(box1);
	hlayout3->addWidget(box2);

    // signals and slots connections
    connect( buttonOk, SIGNAL(clicked()), this, SLOT(quit() ) );
    connect( buttonCancel, SIGNAL(clicked()), this, SLOT(close()));
	connect( buttonApply, SIGNAL(clicked() ), this, SLOT(acceptParams() ) );
	connect( btnWorksheet, SIGNAL(clicked()), this, SLOT(showWorksheet()));
	connect( btnAssociations, SIGNAL(clicked()), this, SLOT(showPlotAssociations()));
	connect( btnEditFunction, SIGNAL(clicked()), this, SLOT(editFunctionCurve()));

	connect(listBox, SIGNAL(doubleClicked (QListBoxItem *)),
			this, SLOT(showPlotAssociations(QListBoxItem *)));
	connect(listBox, SIGNAL(highlighted(int)), this, SLOT(updateTabWindow(int)));
	connect(listBox, SIGNAL(rightButtonClicked(QListBoxItem *, const QPoint &)), this, SLOT(showPopupMenu(QListBoxItem *, const QPoint &)));
	connect(boxPlotType, SIGNAL(activated(int)), this, SLOT(changePlotType(int)));

	QAccel *accel = new QAccel(this);
	accel->connectItem( accel->insertItem( Key_Delete ), this, SLOT(removeSelectedCurve()) );
}

void plotDialog::showPlotAssociations(QListBoxItem *item)
{
if (!item)
	return;

ApplicationWindow *app = (ApplicationWindow *)this->parent();
if (!app)
	return;

QString text = item->text();
if (text.contains("="))
	app->showFunctionDialog(text, listBox->index(item));
else
	app->showPlotAssociations(listBox->index(item));
close();
}

void plotDialog::showPlotAssociations()
{
ApplicationWindow *app = (ApplicationWindow *)this->parent();
if (app)
	app->showPlotAssociations(listBox->currentItem());
}

void plotDialog::changePlotType(int plotType)
{
if (boxPlotType->count() == 1 || (graph->curveType(listBox->currentItem()) == plotType))
	return;

clearTabWidget();
insertTabs(plotType);

lastSelectedCurve = listBox->currentItem();
graph->setCurveType(lastSelectedCurve, plotType);	
setActiveCurve(lastSelectedCurve);

boxConnect->setCurrentItem(1);//show line for Line and LineSymbol plots
acceptParams();
}

void plotDialog::initLinePage()
{
    linePage = new QWidget( privateTabWidget, "linePage" );

	GroupBox3 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),linePage, "GroupBox3" );

	new QLabel( tr( "Connect" ),GroupBox3, "TextLabel1",0 );  

    boxConnect = new QComboBox( FALSE, GroupBox3, "boxConnect" );
	boxConnect->insertItem(tr("No line"));
    boxConnect->insertItem(tr("Lines"));
    boxConnect->insertItem(tr("Sticks"));
    boxConnect->insertItem(tr("Steps"));
    boxConnect->insertItem(tr("Dots"));
	boxConnect->insertItem(tr("Spline"));
	
	new QLabel(tr( "Style" ) , GroupBox3, "TextLabel2",0 );

    boxLineStyle = new QComboBox( FALSE, GroupBox3, "boxLineStyle" );
    boxLineStyle->insertItem("_____");
	boxLineStyle->insertItem("_ _ _");
	boxLineStyle->insertItem(".....");
	boxLineStyle->insertItem("_._._");
	boxLineStyle->insertItem("_.._..");

    new QLabel(tr( "Width" ), GroupBox3, "TextLabel3",0 );  

    boxLineWidth = new QSpinBox( GroupBox3, "boxLineWidth" );
    boxLineWidth->setMinValue( 1 );
	boxLineWidth->setValue( 1 );

    new QLabel( tr( "Color" ), GroupBox3, "TextLabel4",0 );
    boxLineColor = new ColorBox( FALSE, GroupBox3);
	
	fillGroupBox = new QButtonGroup(2,QGroupBox::Horizontal, QString::null, linePage, "fillGroupBox" );

	boxFill = new QCheckBox(fillGroupBox, 0);
	new QLabel(  tr( "Fill area under curve" ),fillGroupBox, "TextLabel4_3",0 );  

	TextLabel4_3=new QLabel(  tr( "Fill color" ),fillGroupBox, "TextLabel4_3",0 );  
    boxAreaColor = new ColorBox( FALSE,fillGroupBox);

	TextLabel4_4=new QLabel(tr( "Pattern" ), fillGroupBox, "TextLabel4_4",0);
	boxPattern = new PatternBox( FALSE, fillGroupBox);
	
	QHBoxLayout* hlayout2 = new QHBoxLayout(linePage,5,5, "hlayout2");
    hlayout2->addWidget(GroupBox3);
	hlayout2->addWidget(fillGroupBox);

	privateTabWidget->insertTab( linePage, tr( "Line" ) );

    //line page connections
	connect(boxLineColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxConnect, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxLineStyle, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxAreaColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxPattern, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxFill, SIGNAL(toggled(bool)), this, SLOT(showAreaColor(bool)));
	connect(boxFill, SIGNAL(clicked()), this, SLOT(acceptParams()));
}

void plotDialog::initSymbolsPage()
{	
    symbolPage = new QWidget( privateTabWidget, "symbolPage" );

	GroupBox0 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),symbolPage, "GroupBox0" );
	
	new QLabel(tr( "Style" ), GroupBox0, "TextLabel2_2",0 );
    boxSymbolStyle = new QComboBox( FALSE, GroupBox0, "boxSymbolStyle" );
	boxSymbolStyle->insertItem(tr("Ellipse" ));
    boxSymbolStyle->insertItem(tr("Rectangle"));
    boxSymbolStyle->insertItem(tr("Diamond"));
    boxSymbolStyle->insertItem(tr("Up Triangle"));
    boxSymbolStyle->insertItem(tr("Down Triangle"));
    boxSymbolStyle->insertItem(tr("Left Triangle"));
    boxSymbolStyle->insertItem(tr("Right Triangle"));
    boxSymbolStyle->insertItem(tr("Cross"));
    boxSymbolStyle->insertItem(tr("Diagonal Cross"));

    new QLabel(tr( "Size" ), GroupBox0, "TextLabel3_2",0 );
	boxSymbolSize = new QSpinBox( GroupBox0, "boxSymbolSize" );
    boxSymbolSize->setMinValue( 1 );
	boxSymbolSize->setValue(5);

	new QLabel( tr( "Fill Color" ), GroupBox0, "TextLabel5_2",0 );
	boxFillColor = new ColorBox( FALSE, GroupBox0);

	GroupBox1 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "Pen" ),symbolPage, "GroupBox0" );

	new QLabel(tr( "Color" ), GroupBox1, "TextLabel4_2",0 ); 
    boxSymbolColor = new ColorBox( FALSE, GroupBox1);
	
	new QLabel(tr( "Width" ), GroupBox1, "TextLabel4_2",0 ); 
    boxPenWidth = new QSpinBox(0, 100, 1, GroupBox1, "boxPenWidth" );

	QHBoxLayout* hlayout0 = new QHBoxLayout(symbolPage,5,25, "hlayout");
    hlayout0->addWidget(GroupBox0);
	hlayout0->addWidget(GroupBox1);

	privateTabWidget->insertTab(symbolPage, tr( "Symbol" ) );

    //symbol page connections
	connect(boxSymbolColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxSymbolStyle, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxFillColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
}

void plotDialog::initErrorsPage()
{
errorsPage = new QWidget(privateTabWidget, "errorsPage" );
	
GroupBox2 = new QButtonGroup(1,QGroupBox::Horizontal,tr( "Direction" ),errorsPage, "GroupBox2" );

plusBox = new QCheckBox( GroupBox2, "plusBox" );
plusBox->setText( tr( "Plus" ) );
plusBox->setChecked( TRUE );

    minusBox = new QCheckBox( GroupBox2, "minusBox" );
    minusBox->setText( tr( "Minus" ) );
    minusBox->setChecked( TRUE );

    xBox = new QCheckBox( GroupBox2, "xBox" ); 
    xBox->setText( tr( "&X Error Bar" ) );

    GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "Style" ),errorsPage, "GroupBox1" );
   
    new QLabel(tr( "Color" ), GroupBox1, "TextLabel3_3",0 );
    colorBox = new ColorButton(GroupBox1);
	colorBox->setText(tr("Co&lor"));

	new QLabel(tr( "Line Width" ), GroupBox1, "TextLabel3_3_2",0 );
    widthBox = new QComboBox( FALSE, GroupBox1, "widthBox" );
	widthBox->insertItem( tr( "1" ) );
    widthBox->insertItem( tr( "2" ) );
    widthBox->insertItem( tr( "3" ) );
    widthBox->insertItem( tr( "4" ) );
    widthBox->insertItem( tr( "5" ) );
	widthBox->setEditable (TRUE);
	widthBox->setCurrentItem(1);
	
	new QLabel(tr( "Cap Width" ) , GroupBox1, "TextLabel3_3_2_2",0 );

    capBox = new QComboBox( FALSE, GroupBox1, "capBox" );
	capBox->insertItem( tr( "8" ) );
    capBox->insertItem( tr( "10" ) );
    capBox->insertItem( tr( "12" ) );
    capBox->insertItem( tr( "16" ) );
    capBox->insertItem( tr( "20" ) );
	capBox->setEditable (TRUE);
	capBox->setCurrentItem(0);

    throughBox = new QCheckBox( GroupBox1, "throughBox" );
    throughBox->setText( tr( "Through Symbol" ) );
	
	QHBoxLayout* hlayout = new QHBoxLayout(errorsPage,5,5, "hlayout");
    hlayout->addWidget(GroupBox2);
	hlayout->addWidget(GroupBox1);

	privateTabWidget->insertTab( errorsPage, tr( "Error Bars" ) );

   	connect(colorBox, SIGNAL(clicked()), this, SLOT(pickErrorBarsColor()));
	connect(xBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsType()));
	connect(plusBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsPlus()));
	connect(minusBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsMinus()));
	connect(throughBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsThrough()));
}

void plotDialog::initHistogramPage()
{
histogramPage = new QWidget(privateTabWidget, "histogramPage" );
	
QHBox *GroupBoxH1 = new QHBox ( histogramPage);
GroupBoxH1->setSpacing (5);
	
automaticBox = new QCheckBox( GroupBoxH1, "automaticBox" );
automaticBox->setText( tr( "Automatic Binning" ) );

buttonStatistics = new QPushButton(GroupBoxH1, "buttonStatistics" );
buttonStatistics->setText( tr( "&Show statistics" ) );

GroupBoxH = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),histogramPage, "GroupBoxH" );
   
new QLabel(tr( "Bin Size" ), GroupBoxH, "TextLabel3_3",0 );
binSizeBox = new QLineEdit(GroupBoxH);	

new QLabel(tr( "Begin" ), GroupBoxH, "TextLabel3_3_2",0 );
histogramBeginBox = new QLineEdit(GroupBoxH);	

new QLabel(tr( "End" ) , GroupBoxH, "TextLabel3_3_2_2",0 );
histogramEndBox = new QLineEdit(GroupBoxH);
	
QVBoxLayout* hlayout = new QVBoxLayout(histogramPage,5,5, "hlayout");
hlayout->addWidget(GroupBoxH1);
hlayout->addWidget(GroupBoxH);

connect(automaticBox, SIGNAL(clicked()), this, SLOT(setAutomaticBinning()));
connect(buttonStatistics, SIGNAL(clicked()), this, SLOT(showStatistics() ) );

privateTabWidget->insertTab( histogramPage, tr( "Histogram Data" ) );
}

void plotDialog::initSpacingPage()
{
spacingPage = new QWidget(privateTabWidget, "histogramPage" );

QHBox *hb1 = new QHBox ( spacingPage);
hb1->setSpacing (5);

new QLabel(tr( "Gap Between Bars (in %)" ) , hb1, "TextLabel_gap",0 );
gapBox=new QSpinBox(0,100,10,hb1, "gapBox");

QHBox *hb2 = new QHBox ( spacingPage);
hb2->setSpacing (5);

new QLabel(tr( "Offset (in %)" ) , hb2, "TextLabel_gap",0 );
offsetBox=new QSpinBox(-1000,1000,50,hb2, "gapBox");

QVBoxLayout* hlayout = new QVBoxLayout(spacingPage, 35, 5, "hlayout");
hlayout->addWidget(hb1);
hlayout->addWidget(hb2);

privateTabWidget->insertTab( spacingPage, tr( "Spacing" ) );
}

void plotDialog::initVectPage()
{
vectPage = new QWidget(privateTabWidget, "vectPage" );
QVBox *box1 = new QVBox(vectPage);

QButtonGroup *GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "" ),box1, "GroupBox1" );
	
new QLabel(tr( "Color" ), GroupBox1, "TextLabel3_3",0 );
vectColorBox = new ColorBox( FALSE, GroupBox1);

new QLabel(tr( "Line Width" ), GroupBox1, "TextLabel3_3_2",0 );
vectWidthBox = new QSpinBox( 1,100,1, GroupBox1, "vectWidthBox" );

GroupBox2 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "Arrowheads" ),box1, "GroupBox2" );

new QLabel(tr( "Length" ), GroupBox2, "TextLabel3_3",0 );
headLengthBox = new QSpinBox( 0,100,1, GroupBox2, "headLengthBox" );
	
new QLabel(tr( "Angle" ), GroupBox2, "TextLabel3_3_2",0 );
headAngleBox = new QSpinBox( 0,85,5, GroupBox2, "headAngleBox" );

filledHeadBox = new QCheckBox(GroupBox2, "filledBox" ); 
filledHeadBox->setText( tr( "&Filled" ) );

QVBox *box2 = new QVBox(vectPage);
GroupBox3 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "End Point" ),box2, "GroupBox2" );

new QLabel(tr( "X End" ), GroupBox3, "TextLabel3_3",0 );
xEndBox = new QComboBox( FALSE, GroupBox3, "headLengthBox" );
	
new QLabel(tr( "Y End" ), GroupBox3, "TextLabel3_3_2",0 );
yEndBox = new  QComboBox( FALSE, GroupBox3, "headAngleBox" );

QHBoxLayout* hlayout = new QHBoxLayout(vectPage,5,5, "hlayout");
hlayout->addWidget(box1);
hlayout->addWidget(box2);

privateTabWidget->insertTab( vectPage, tr( "Vector" ) );
}

void plotDialog::setGraph(Graph *g)
{
graph = g;
insertCurvesList();

connect( graph, SIGNAL(modifiedFunction()), this, SLOT(insertCurvesList() ) );
connect( graph, SIGNAL( modifiedPlotAssociation() ), this, SLOT( insertCurvesList() ) );
}

void plotDialog::selectCurve(int index)
{
insertTabs(graph->curveType(index));
lastSelectedCurve = index;
listBox->setSelected (index,TRUE);
setActiveCurve(index);
}

void plotDialog::showStatistics()
{
QString text=listBox->currentText();
QStringList t=QStringList::split(": ", text, false);
QStringList list=QStringList::split (",", t[1], FALSE );
text=t[0] + "_" + list[1].remove("(Y)");
	
ApplicationWindow *app = (ApplicationWindow *)this->parent();
if (app)
	{
	Table* w=app->table(text);
	if (!w)
		return;
	QString result=graph->showHistogramStats(w, text, listBox->currentItem());
	if (!result.isEmpty())
		{
		app->logInfo+=result;
		app->showResults(true);
		}
	}
close();
}

void plotDialog::changeCurveLayout(int)
{
if (privateTabWidget->currentPage()==linePage || privateTabWidget->currentPage()==symbolPage)
	{		
	int index=listBox->currentItem();
	curveLayout cl = getLayout();
	graph->updateCurveLayout(index,&cl);
	graph->replot();
	}
}

void plotDialog::showPopupMenu(QListBoxItem *it, const QPoint &point)
{
if (!it)
	return;

lastSelectedCurve=listBox->index (it);

QPopupMenu contextMenu(this);
if (listBox->count() > 1)
	contextMenu.insertItem(tr("&Delete"), this, SLOT(removeCurve()));
if (it->text().contains("="))
	contextMenu.insertItem(tr("&Edit..."), this, SLOT(editFunctionCurve()));
else
	contextMenu.insertItem(tr("&Plot Associations..."), this, SLOT(showPlotAssociations()));

contextMenu.exec(point);
}

void plotDialog::editFunctionCurve()
{
int curve = listBox->currentItem();
ApplicationWindow *app = (ApplicationWindow *)this->parent();
if (app)
	app->showFunctionDialog(listBox->text(curve), curve);
}

void plotDialog::removeCurve()
{
graph->removeCurve(lastSelectedCurve);
listBox->removeItem (lastSelectedCurve);
}

void plotDialog::removeSelectedCurve()
{
int curve=listBox->currentItem ();
graph->removeCurve(curve);
listBox->removeItem (curve);
}

void plotDialog::changeErrorBarsPlus()
{
graph->updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(),colorBox->color(), plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void plotDialog::changeErrorBarsMinus()
{
graph->updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(), colorBox->color(),plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void plotDialog::changeErrorBarsThrough()
{
graph->updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(), colorBox->color(),plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void plotDialog::changeErrorBarsType()
{
graph->updateErrorBars(listBox->currentItem(), xBox->isChecked(), widthBox->currentText().toInt(),
					 capBox->currentText().toInt(), colorBox->color(), plusBox->isChecked(), minusBox->isChecked(),
					 throughBox->isChecked());
}

void plotDialog::pickErrorBarsColor()
{
QColor color = QColorDialog::getColor(colorBox->color(), this);
if ( !color.isValid() || color == colorBox->color() )
	return;

colorBox->setColor (color) ;
	
graph->updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(), color, plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void plotDialog::showAreaColor(bool show)
{
boxAreaColor->setEnabled(show);
TextLabel4_3->setEnabled(show);
TextLabel4_4->setEnabled(show);
boxPattern->setEnabled(show);
}

void plotDialog::updateTabWindow(int curveIndex)
{
int plot_type = setPlotType(curveIndex);
if (plot_type < 0)
	return;
else if (graph->curveType(lastSelectedCurve) == plot_type)
	{
	setActiveCurve(curveIndex);
	return;
	}

clearTabWidget();

QString text=listBox->currentText();
if (text.contains("="))
	{
	btnAssociations->hide();
	btnEditFunction->show();
	}
else
	{
	btnAssociations->show();
	btnEditFunction->hide();
	}

insertTabs(plot_type);
setActiveCurve(curveIndex);
lastSelectedCurve = curveIndex;
}

void plotDialog::insertTabs(int plot_type)
{
if (plot_type == Graph::Line)
	{
	boxConnect->setEnabled(true);
	privateTabWidget->addTab (linePage, tr("Line"));
	privateTabWidget->showPage(linePage);
	}
else if (plot_type == Graph::Scatter)
	{
	boxConnect->setEnabled(true);
	privateTabWidget->addTab (symbolPage, tr("Symbol"));
	privateTabWidget->showPage(symbolPage);
	}
else if (plot_type == Graph::LineSymbols)
	{
	boxConnect->setEnabled(true);
	privateTabWidget->addTab (linePage, tr("Line"));
	privateTabWidget->addTab (symbolPage, tr("Symbol"));
	privateTabWidget->showPage(symbolPage);
	}
else if (plot_type == Graph::VerticalBars || 
		 plot_type == Graph::HorizontalBars || 
		 plot_type == Graph::Histogram)
	{
	boxConnect->setEnabled(false);
	privateTabWidget->addTab (linePage, tr("Pattern"));
	privateTabWidget->addTab (spacingPage, tr("Spacing"));

	if (plot_type == Graph::Histogram)
		{
		privateTabWidget->addTab (histogramPage, tr("Histogram Data"));	
		privateTabWidget->showPage(histogramPage);
		}
	else
		privateTabWidget->showPage(linePage);
	}
else if (plot_type == Graph::VectXYXY)
	{
	boxConnect->setEnabled(true);
	privateTabWidget->addTab (linePage, tr("Line"));
	privateTabWidget->addTab (vectPage, tr("Vector"));
	privateTabWidget->showPage(vectPage);
	}
else if (plot_type == Graph::ErrorBars)
	{
	privateTabWidget->addTab (errorsPage, "Error Bars");
	privateTabWidget->showPage(errorsPage);
	}
}

void plotDialog::clearTabWidget()
{
privateTabWidget->removePage(linePage);
privateTabWidget->removePage(symbolPage);
privateTabWidget->removePage(errorsPage);
privateTabWidget->removePage(histogramPage);
privateTabWidget->removePage(spacingPage);
privateTabWidget->removePage(vectPage);
}

void plotDialog::quit()
{
if (acceptParams())
	close();
}

void plotDialog::showWorksheet()
{
int index=listBox->currentItem();
QString text=listBox->text(index);
if (text.contains("="))
	graph->createWorksheet(text);
else
	{
	ApplicationWindow *app = (ApplicationWindow *)this->parent();
	if (!app)
		return;

	QStringList t=QStringList::split(": ", text, false);
	app->showTable(t[0]+"_");
	}
close();
}

int plotDialog::setPlotType(int index)
{
int size=listBox->count();
int curveType = graph->curveType(index);
if (size>0 && curveType >= 0)
	{
	boxPlotType->clear();

	if (curveType == Graph::ErrorBars)
		boxPlotType->insertItem( tr( "Error Bars" ) );
	else if (curveType == Graph::VerticalBars)
		boxPlotType->insertItem( tr( "Vertical Bars" ) );
	else if (curveType == Graph::HorizontalBars)
		boxPlotType->insertItem( tr( "Horizontal Bars" ) );
	else if (curveType == Graph::Histogram)
		boxPlotType->insertItem( tr( "Histogram" ) );
	else if (curveType == Graph::VectXYXY)
		boxPlotType->insertItem( tr( "Vector XYXY" ) );
	else 
		{
		boxPlotType->insertItem( tr( "Line" ) );
		boxPlotType->insertItem( tr( "Scatter" ) );
		boxPlotType->insertItem( tr( "Line + Symbol" ) ); 

		if (curveType == Graph::Line || 
			curveType == Graph::Steps || 
			curveType == Graph::Area)
			{
			boxPlotType->setCurrentItem(0);
			return Graph::Line;
			}
		else if (curveType == Graph::Scatter)
			boxPlotType->setCurrentItem(1);
		else if (curveType == Graph::LineSymbols || 
			     curveType == Graph::VerticalDropLines || 
				 curveType == Graph::Spline)
			{
			boxPlotType->setCurrentItem(2);
			return Graph::LineSymbols;
			}
		}
	}
return curveType;
}

void plotDialog::setActiveCurve(int index)
{//connected to the listBox highlighted signal, displays the current curve parameters
int size=listBox->count();
if (size>0)
	{
	int curveType = graph->curveType(index);

	QwtPlotCurve *c = (QwtPlotCurve*)graph->curve(index);
	if (!c)
		return;

	//line page
	boxConnect->setCurrentItem(c->style());
	setPenStyle(c->pen().style());
	boxLineColor->setColor(c->pen().color());
	boxLineWidth->setValue(c->pen().width());
		
	boxFill->setChecked(c->brush().style() != Qt::NoBrush );
	boxAreaColor->setColor(c->brush().color());
	boxPattern->setPattern(c->brush().style());

		
	//symbol page
	const QwtSymbol s = c->symbol();
	boxSymbolSize->setValue(s.size().width());
	setSymbolStyle(s.style());
	boxSymbolColor->setColor(s.pen().color());
	boxPenWidth->setValue(s.pen().width());
	boxFillColor->setColor(s.brush().color());
			
	if (curveType == Graph::VerticalBars || curveType == Graph::HorizontalBars || 
		curveType == Graph::Histogram)
		{//spacing page
		QwtBarCurve *b = (QwtBarCurve*)graph->curve(index);
		if (b)
			{
			gapBox->setValue(b->gap());
			offsetBox->setValue(b->offset());
			}
		}

	if (curveType == Graph::Histogram)
		{//Histogram page
		QwtHistogram *h = (QwtHistogram*)graph->curve(index);
		if (h)
			{
			automaticBox->setChecked(h->autoBinning());
			binSizeBox->setText(QString::number(h->binSize()));
			histogramBeginBox->setText(QString::number(h->begin()));
			histogramEndBox->setText(QString::number(h->end()));
			setAutomaticBinning();
			}
		}
	
	if (curveType == Graph::VectXYXY)
		{//Vector page		
		VectXYXYCurve *v = (VectXYXYCurve*)graph->curve(index);
		if (v)
			{
			vectColorBox->setColor(v->color());
			vectWidthBox->setValue(v->width());
			headLengthBox->setValue(v->headLength());
			headAngleBox->setValue(v->headAngle());
			filledHeadBox->setChecked(v->filledArrowHead());

			updateEndPointColumns(listBox->currentText());
			}
		}

	if (curveType == Graph::ErrorBars)
		{
		QwtErrorPlotCurve *err = (QwtErrorPlotCurve*)graph->curve(index);
		if (err)
			{
			xBox->setChecked(err->xErrors());
			widthBox->setEditText(QString::number(err->width()));
			capBox->setEditText(QString::number(err->capLength()));
			colorBox->setColor(err->color());
			throughBox->setChecked(err->throughSymbol());
			plusBox->setChecked(err->plusSide());
			minusBox->setChecked(err->minusSide());
			}
		}
	}
}

void plotDialog::updateEndPointColumns(const QString& text)
{
QStringList cols=QStringList::split(",", text, false);
QStringList aux=QStringList::split(":", cols[0], false);
QString table=aux[0];	
QStringList list;
for (int i=0; i<(int)columnNames.count(); i++)	
	{
	QString s=columnNames[i];
	if (s.contains(table))
		list<<s;	
	}
xEndBox->clear();
xEndBox->insertStringList(list);
xEndBox->setCurrentText(table + "_" + cols[2].remove("(X)"));
	
yEndBox->clear();
yEndBox->insertStringList(list);	
yEndBox->setCurrentText(table + "_" + cols[3].remove("(Y)"));	
}

curveLayout plotDialog::getLayout()
{
curveLayout cl;

cl.connectType=boxConnect->currentItem();
cl.lStyle=boxLineStyle->currentItem();
cl.lWidth=boxLineWidth->value();

if (boxFill->isChecked()) 
	cl.filledArea=1;
else 
	cl.filledArea=0;

cl.aCol=boxAreaColor->currentItem();
cl.aStyle=boxPattern->currentItem();

cl.sSize=boxSymbolSize->value();
cl.sType=boxSymbolStyle->currentItem();
cl.symCol=boxSymbolColor->currentItem();
cl.penWidth = boxPenWidth->value();
cl.fillCol=boxFillColor->currentItem();
cl.lCol=boxLineColor->currentItem();
return cl;
}

bool plotDialog::acceptParams()
{
int index=listBox->currentItem();
if (privateTabWidget->currentPage()==linePage || privateTabWidget->currentPage()==symbolPage)
	{
	curveLayout cl = getLayout();
	graph->updateCurveLayout(index,&cl);
	graph->replot();
	}
else if (privateTabWidget->currentPage()==histogramPage)
	{
	QString text=listBox->currentText();
	QStringList t=QStringList::split(": ", text, false);
	QStringList list=QStringList::split (",", t[1], FALSE );
	text=t[0] + "_" + list[1].remove("(Y)");
	bool accept=validInput();
	if (accept)
		{
		ApplicationWindow *app = (ApplicationWindow *)this->parent();
		if (!app)
			return false;
		Table* w=app->table(text);
		if (w)
			graph->updateHistogram(w, text, listBox->currentItem(), automaticBox->isChecked(),binSizeBox->text().toDouble(), 
						 histogramBeginBox->text().toDouble(), histogramEndBox->text().toDouble());		
		}
	return accept;
	}
else if (privateTabWidget->currentPage()==spacingPage)
	{
	graph->setBarsGap(listBox->currentItem(),gapBox->value(), offsetBox->value());
	graph->replot();
	return true;
	}
else if (privateTabWidget->currentPage()==vectPage)
	{
	int index=listBox->currentItem();
	ApplicationWindow *app = (ApplicationWindow *)this->parent();
	if (!app)
		return false;
	Table* w=app->table(xEndBox->currentText());
	if (!w)
		return false;

	graph->updateVectorsLayout(w, index, vectColorBox->currentItem(), vectWidthBox->value(), 
							headLengthBox->value(), headAngleBox->value(), filledHeadBox->isChecked(),
							xEndBox->currentText(), yEndBox->currentText());
	
	QString text=listBox->currentText();
	QStringList t=QStringList::split(": ", text, false);
	QString table = t[0];
		
	QStringList cols=QStringList::split(",", t[1], false);
	QString xEndCol=xEndBox->currentText().remove(table + "_") + "(X)";
	QString yEndCol=yEndBox->currentText().remove(table + "_") + "(Y)";
	if (cols[2] != xEndCol || cols[3] != yEndCol)
		{
		cols[2] = xEndCol;
		cols[3] = yEndCol;
		text=table + ": " + cols.join(",");
		listBox->changeItem (text, index);
		}		
	return true;
	}
else if (privateTabWidget->currentPage() == errorsPage)
	{
	graph->updateErrorBars(listBox->currentItem(), xBox->isChecked(), widthBox->currentText().toInt(),
					 capBox->currentText().toInt(), colorBox->color(), plusBox->isChecked(), minusBox->isChecked(),
					 throughBox->isChecked());
	}
return true;
}

void plotDialog::insertCurvesList()
{
QStringList names = graph->plotAssociations();
listBox->clear();
QStringList newNames;
int i;
for (i=0;i<(int)names.count();i++)
	{
	QString s=names[i];
	int pos=s.find("_",0);
	if (pos>0)
		{
		QString table=s.left(pos);	
		QString cols=s.right(s.length()-pos-1);			
		newNames<<table+": "+cols.remove(table+"_",TRUE);
		}
	else
		newNames<<s;
	}
listBox->insertStringList(newNames,0);
}

void plotDialog::setAutomaticBinning()
{
GroupBoxH->setEnabled(!automaticBox->isChecked());
}

bool plotDialog::validInput()
{
QString from=histogramBeginBox->text();
QString to=histogramEndBox->text();
QString step=binSizeBox->text();
QRegExp nonDigit("\\D");
		
if (histogramBeginBox->text().isEmpty())
	{
	QMessageBox::critical(0,tr("QtiPlot - Input error"),
				tr("Please enter a valid start limit!"));
	histogramBeginBox->setFocus();
	return FALSE;
	}
		
if (histogramEndBox->text().isEmpty())
		{
		QMessageBox::critical(0,tr("QtiPlot - Input error"),
				tr("Please enter a valid end limit!"));
		histogramEndBox->setFocus();
		return FALSE;
		}
		
if (binSizeBox->text().isEmpty())
		{
		QMessageBox::critical(0,tr("QtiPlot - Input error"),
				tr("Please enter a valid bin size value!"));
		binSizeBox->setFocus();
		return FALSE;
		}	
		
from=from.remove(".");
to=to.remove(".");
step=step.remove(".");

int pos=from.find("-",0);
if(pos==0)
		from=from.replace(pos,1,"");

pos=to.find("-",0);
if(pos==0)
	to=to.replace(pos,1,"");
	
double start,end, stp;
bool error=FALSE;	
if (from.contains(nonDigit))
		{
		try
			{
			myParser parser;
			parser.SetExpr((histogramBeginBox->text()).ascii());
			start=parser.Eval();
			}
		catch(mu::ParserError &e)
			{
			QMessageBox::critical(0,"QtiPlot - Start limit error",e.GetMsg());
			histogramBeginBox->setFocus();
			error=TRUE;
			return FALSE;
			}	
		}
		else
			start=histogramBeginBox->text().toDouble();
		
		if (to.contains(nonDigit))
			{
			try
				{
				myParser parser;
				parser.SetExpr((histogramEndBox->text()).ascii());
				end=parser.Eval();
				}
			catch(mu::ParserError &e)
				{
				QMessageBox::critical(0,"QtiPlot - End limit error",e.GetMsg());
				histogramEndBox->setFocus();
				error=TRUE;
				return FALSE;
				}	
			}
		else
			end=histogramEndBox->text().toDouble();

		if (start>=end)
			{
			QMessageBox::critical(0,tr("QtiPlot - Input error"),
					tr("Please enter limits that satisfy: begin < end!"));
			histogramEndBox->setFocus();
			return FALSE;
			}
			
		if (step.contains(nonDigit))
		{
		try
			{
			myParser parser;
			parser.SetExpr((binSizeBox->text()).ascii());
			stp=parser.Eval();
			}
		catch(mu::ParserError &e)
			{
			QMessageBox::critical(0,tr("QtiPlot - Bin size input error"),e.GetMsg());
			binSizeBox->setFocus();
			error=TRUE;
			return FALSE;
			}	
		}
		else
			stp=binSizeBox->text().toDouble();
		
		if (stp <=0)
			{
			QMessageBox::critical(0,tr("QtiPlot - Bin size input error"),tr("Please enter a positive bin size value!"));
			binSizeBox->setFocus();
			return FALSE;	
			}

return TRUE;
}

void plotDialog::setPenStyle(Qt::PenStyle style)
{
switch (style)
	{
	case Qt::SolidLine:
		boxLineStyle->setCurrentItem(0);
	break;
	case Qt::DashLine:
		boxLineStyle->setCurrentItem(1);
	break;
	case Qt::DotLine:
		boxLineStyle->setCurrentItem(2);
	break;
	case Qt::DashDotLine:
		boxLineStyle->setCurrentItem(3);
	break;
	case Qt::DashDotDotLine:
		boxLineStyle->setCurrentItem(4);
	break;
	}
}

void plotDialog::setSymbolStyle(QwtSymbol::Style style)
{
switch (style)
	{
	case QwtSymbol::Ellipse:
		boxSymbolStyle->setCurrentItem(0); 
	break;
	case QwtSymbol::Rect:
		boxSymbolStyle->setCurrentItem(1);
	break;
	case QwtSymbol::Diamond:
		boxSymbolStyle->setCurrentItem(2);
	break;
	case QwtSymbol::Triangle:
		boxSymbolStyle->setCurrentItem(3);
	break;
	case QwtSymbol::DTriangle:
		boxSymbolStyle->setCurrentItem(4);
	break;
	case QwtSymbol::LTriangle:
		boxSymbolStyle->setCurrentItem(5);
	break;
	case QwtSymbol::RTriangle:
		boxSymbolStyle->setCurrentItem(6);
	break;
	case QwtSymbol::Cross:
		boxSymbolStyle->setCurrentItem(7);
	break;
	case QwtSymbol::XCross:
		boxSymbolStyle->setCurrentItem(8);
	break;
	}
}

plotDialog::~plotDialog()
{
}
